@@ -734,7 +734,10 @@ win_redr_ruler(win_T *wp, int always, int ignore_pum)
 	    wp->w_p_list = TRUE;
 	}
 
-	bufferlen = vim_snprintf((char *)buffer, RULER_BUF_LEN, "%ld,",
+	// row number, column number is appended
+	// l10n: leave as-is unless a space after the comma is preferred
+	// l10n: do not add any row/column label, due to the limited space
+	bufferlen = vim_snprintf((char *)buffer, RULER_BUF_LEN, _("%ld,"),
 		(wp->w_buffer->b_ml.ml_flags & ML_EMPTY)
 		    ? 0L
 		    : (long)(wp->w_cursor.lnum));
