@@ -10,19 +10,12 @@
  * os_unixx.h -- include files that are only used in os_unix.c
  */
 
-/*
- * Stuff for signals
- */
-#if defined(HAVE_SIGSET) && !defined(signal)
-# define signal sigset
-#endif
-
-   /* Sun's sys/ioctl.h redefines symbols from termio world */
+// Sun's sys/ioctl.h redefines symbols from termio world
 #if defined(HAVE_SYS_IOCTL_H) && !defined(SUN_SYSTEM)
 # include <sys/ioctl.h>
 #endif
 
-#ifndef USE_SYSTEM	/* use fork/exec to start the shell */
+#ifndef USE_SYSTEM	// use fork/exec to start the shell
 
 # if defined(HAVE_SYS_WAIT_H) || defined(HAVE_UNION_WAIT)
 #  include <sys/wait.h>
@@ -44,16 +37,16 @@
 #  endif
 # endif
 
-#endif /* !USE_SYSTEM */
+#endif // !USE_SYSTEM
 
 #ifdef HAVE_STROPTS_H
-#ifdef sinix
-#define buf_T __system_buf_t__
-#endif
+# ifdef sinix
+#  define buf_T __system_buf_t__
+# endif
 # include <stropts.h>
-#ifdef sinix
-#undef buf_T
-#endif
+# ifdef sinix
+#  undef buf_T
+# endif
 #endif
 
 #ifdef HAVE_STRING_H
@@ -69,12 +62,12 @@
 #endif
 
 #ifdef HAVE_SYS_SYSTEMINFO_H
-/* <sys/systeminfo.h> uses SYS_NMLN but it may not be defined (CrayT3E). */
+// <sys/systeminfo.h> uses SYS_NMLN but it may not be defined (CrayT3E).
 # ifndef SYS_NMLN
 #  define SYS_NMLN 32
 # endif
 
-# include <sys/systeminfo.h>	/* for sysinfo */
+# include <sys/systeminfo.h>	// for sysinfo
 #endif
 
 /*
@@ -96,16 +89,16 @@
 #endif
 
 #ifdef HAVE_SYS_PTEM_H
-# include <sys/ptem.h>	/* must be after termios.h for Sinix */
-# ifndef _IO_PTEM_H	/* For UnixWare that should check for _IO_PT_PTEM_H */
+# include <sys/ptem.h>	// must be after termios.h for Sinix
+# ifndef _IO_PTEM_H	// For UnixWare that should check for _IO_PT_PTEM_H
 #  define _IO_PTEM_H
 # endif
 #endif
 
-/* shared library access */
+// shared library access
 #if defined(HAVE_DLFCN_H) && defined(USE_DLOPEN)
 # if defined(__MVS__) && !defined (__SUSV3)
-    /* needed to define RTLD_LAZY (Anthony Giorgio) */
+    // needed to define RTLD_LAZY (Anthony Giorgio)
 #  define __SUSV3
 # endif
 # include <dlfcn.h>
